# edlib.pl - Text Editor subroutines
# This file is part of Syrine 0.1b
# Copyright (C) 1997,1998  Yoram Last (ylast@mindless.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

sub DoEditCom { # Handles Text Editor commands.

  local($filename,$edcom) = @_;
  unless ($filename) {$filename = $input{'fname'};};
  unless ($edcom) {$edcom = $input{'edcom'};};
  local($tefname,$ofname,$overwrite,$rflag)  = 
    ($input{'tfname'},$input{'ofname'},$input{'overwrite'},$input{'rflag'});
  $edcom =~ s/\s//g;
  $EdFile = $input{'edfile'};
  $EdFile =~ s/\r//g;
  if ($EdEmptyLnHldr) {
    $EdFile =~ s/^$EdEmptyLnHldr\n/\n/;
    if (!$EdOPFL) {
      $EdFile =~ s/\n$EdEmptyLnHldr\n/\n\n/g;
      $EdFile =~ s/\n$EdEmptyLnHldr\n/\n\n/g;
      $EdFile =~ s/\n$EdEmptyLnHldr$/\n/;
    };
  };
  if ($EdTAMod) {
    $EdFile =~ s/\<$EdTAMod(\/textarea\>)/\<$1/gi;
  };
  if ($edcom eq 'Save') {
    $tefname = &SaveEdFile($filename);
    if ($tefname) {
      &PromptEFName('Save As...',$tefname,$filename,$filename,$overwrite,1);
    } else {
      &GetEdFile($filename);
      $EdFileName = $filename;
      $PrintEd = 'e';
      $output = "The file $filename had been saved";
    };
  } elsif ($edcom =~ m/^SaveAs/) {
    if ($tefname) {
      $tefname = &RenameTEF($tefname,$filename,$ofname,$overwrite,$rflag);
      unless ($tefname) {
        if ($rflag) {
          &GetEdFile($filename);
          $EdFileName = $filename;
          $PrintEd = 'e';
        };
        $output = "The file $filename had been saved";
      };
    } else {
      $tefname = &SaveEdFile;
      &PromptEFName('Save As...',$tefname,$filename,$filename,$overwrite,1);
    };
  } elsif ($edcom eq 'Save+Exit') {
    $tefname = &SaveEdFile($filename);
    if ($tefname) {
      &PromptEFName('Save As...',$tefname,$filename,$filename,$overwrite,0);
    } else {
      $output = "The file $filename had been saved";
    };
  } elsif ($edcom eq 'Reload' || (!$edcom && !$tefname)) {
    &GetEdFile($filename);
    $EdFileName = $filename;
    $PrintEd = 'e';
  } elsif ($edcom eq 'Options') {
    $EdFileName = $filename;
    $PrintEd = &DoEdOptionCom($input{'opcom'});
  } elsif ($edcom eq 'Exit') {
    $output = "Exited Text Editor without saving";
  } elsif ($edcom eq 'Cancel') {
    if (-e $tefname) {
      &GetEdFile($tefname);
      $EdFileName = $ofname;
      $PrintEd = 'e';
      unless (unlink($tefname)) {
        $output = "Can\'t delete the temporary file $tefname \: $!";
      };
    } else {
      $output = "The temporary file $tefname no longer exists";
    };
  } else {
    &SyrineDie("Unknown editor command\: $edcom");
  };
  $output = 'ok' unless ($output);
}

sub SaveEdFile { # Saves $EdFile to a file.

  local($filename) = @_;
  local($tefname) = '';
  unless ($filename) {
    $tefname = join('',$counter,time,'.txt');
    $tefname = &FullPath($TempDir,$tefname);
    $filename = $tefname;
  };
  &GetUmask if ($OStype eq 'U');
  $! = 0;
  open(FILE,">$filename") || &SyrineDie("Can\'t save $filename \: $!");
  print FILE $EdFile;
  close(FILE);
  return $tefname;
}
  
sub GetEdFile { # Reads a text file into $EdFile.

  local($filename) = @_;
  local($buf) = '';
  $EdFile = '';
  if ($filename) {
    $! = 0;
    ((-f $filename) && open(FILE,"<$filename")) ||
      &SyrineDie("Can\'t open $filename \: $!");
    while (read(FILE,$buf,$BufSize)) {
      $EdFile .= $buf;
    };
    close FILE;
  };
}

sub RenameEdFile { # Renames (or moves if renaming is not possible) a file.

  local($oldname,$newname) = @_;
  local($success) = 0;
  if (rename($oldname,$newname)) {
    $success = 1;
  } elsif (!$Win32) {
    (-f $oldname && -r _ && -w _) ||
      &SyrineDie("Can\'t rename the file $oldname \: $!");
    open(OF,"<$oldname") || &SyrineDie("Can\'t read the file $oldname \: $!");
    binmode OF;
    if (open(NF,">$newname")) {
      binmode NF;
      local($buf) = '';
      while (read(OF,$buf,$BufSize)) {
        print NF $buf || &SyrineDie("Can\'t write to the file $newname \: $!");
      };
      close OF; close NF;
      chmod int((stat $oldname)[2]), $newname;
      unlink($oldname);
      $success = 1;
    };
  };
  return $success;
}

sub RenameTEF { # Renames a temporary file.

  local($tefname,$filename,$ofname,$overwrite,$rflag) = @_;
  &SyrineDie("Can\'t find $tefname \: $!") unless (-e $tefname);  
  if ($filename) {
    local($savename) = $filename;
    $savename = &FullPath($pwd,$savename) unless ($savename =~ m/^\// || ($Win32 && $savename =~ m/^\w\:/));
    $! = 0;
    local($FileExists) = -e $filename;
    unless ($FileExists && -d $filename) {
      unless ($FileExists && !$overwrite) {
        if (&RenameEdFile($tefname,$savename)) {
          $tefname = '';
        } else {&PromptEFName('Bad Filename',$tefname,$filename,$ofname,$overwrite,$rflag);};
      } else {&PromptEFName('File Already Exists',$tefname,$filename,$ofname,$overwrite,$rflag);};
    } else {&PromptEFName("$filename is an Existing Directory",$tefname,$filename,$ofname,$overwrite,$rflag);};
  } else {&PromptEFName('No Filename',$tefname,$filename,$ofname,$overwrite,$rflag);};
  return $tefname;
}

sub PromptEFName { # Prompts for a file name in case it is absent or problematic.

  local($problem,$tefname,$filename,$ofname,$overwrite,$rflag) = @_;
  &PrintHTTPHeaders;
  print '<HTML><HEAD><TITLE> Syrine - Text Editor - Name Prompt </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'>',"\n",
  '<H2> Syrine: Text Editor: ',"$problem\: $!",' </H2>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  print '<H4> Please choose one of the following: </H4><CENTER>',"\n",
  '<FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
  '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
  '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
  '<INPUT TYPE=hidden name="com" value="rshedit">',"\n",
  '<INPUT TYPE=hidden name="ofname" value="',"$ofname",'">',"\n",
  '<INPUT TYPE=hidden name="rflag" value="',"$rflag",'">',"\n",
  '<INPUT TYPE=hidden name="tfname" value="',"$tefname",'">',"\n",
  '<TABLE BGCOLOR="',"$TableColor\" $FTableParam",'>',"\n",
  '<TR><TD ALIGN="center">',"\n",
  '<TABLE ',"$TableParam",'><TR><TD ALIGN="center">',"\n",
  '<INPUT TYPE=checkbox name="overwrite" value="1"';
  if ($overwrite) {print ' CHECKED';};
  print "\>\n",'Overwrite existing file',"\n",
  '</TD></TR><TR><TD ALIGN="center">',"\n",
  '<INPUT TYPE=submit name="edcom" value="Save As:">',"\n",
  '<INPUT SIZE="',"$FNFieldSize",'" name="fname" value="',"$filename",'">',"\n",
  '</TD></TR><TR><TD ALIGN="center">',"\n",
  '<INPUT TYPE=submit name="edcom" value="              Cancel               ">',"\n",
  '</TD></TR></TABLE></TD></TR></TABLE></FORM>',"\n",
  '</CENTER><HR></BODY></HTML>',"\n";
  $done = 1;
}

sub DoEdOptionCom { # Handles Text Editor options.

  local($opcom) = @_;
  $opcom =~ s/\s//g;
  local($ScriptFileName) = $ENV{'SCRIPT_FILENAME'};
  $ScriptFileName = $0 unless ($ScriptFileName);
  if ($opcom eq 'OK') {
    require "$ProgDir/lib/oplib.pl" ||
      &SyrineDie("Can\'t require oplib\.pl \: $!");
    &ModOptions($ScriptFileName,'editor');
    &GetEdFile($filename);
    return 'e';
  } elsif ($opcom eq 'Apply') {
    require "$ProgDir/lib/oplib.pl" ||
      &SyrineDie("Can\'t require oplib\.pl \: $!");
    &ModOptions($ScriptFileName,'editor');
    return 'o';
  } elsif ($opcom eq 'Cancel') {
    &GetEdFile($filename);
    return 'e';
  } elsif ($opcom eq 'Help') {
    $EdHelp = 1;
    return 'o';
  } elsif ($opcom eq 'EdTest') {
    $EdTest = 1;
    return 'o';
  } elsif (!$opcom) {
    return 'o';
  } else {
    &SyrineDie("Unknown command\: $opcom");
  };
}

sub GetHelp { # Gets the content of an htm help file.

  local($htmfile,$title) = @_;
  local($content) = &ParseHTM(&GetHTM($htmfile));
  $content =~ s/^[\x00-\xff]*\<HR\>//;
  $title = "<B> $title </B>\n"  if ($title);
  $content = join('',$title,'<HR>',$content);
  return $content;
}

sub PrintEdit { # Prints Editor output.

  if ($PrintEd eq 'e') {
    &PrintEdFile($EdFileName);
  } elsif ($PrintEd eq 'o') {
    &PrintEdOptions($EdFileName);
  } else {
    &PrintCom;
  };
}

sub PrintEdFile { # Prints Text Editor interface.

  local($filename) = @_;
  print '<HTML><HEAD><TITLE> Syrine - Text Editor </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'>',"\n",
  '<B> Syrine: Text Editor: ',"$filename",' </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    if ($output ne 'ok') {print '<HR><PRE>',&HtmlText($output),'</PRE>',"\n";};
    if ($EdEmptyLnHldr) {
      $EdFile =~ s/^([\r\n])/$EdEmptyLnHldr$1/;
      if (!$EdOPFL) {
        $EdFile =~ s/\n([\r\n])/\n$EdEmptyLnHldr$1/g;
        $EdFile =~ s/\n([\r\n])/\n$EdEmptyLnHldr$1/g;
        $EdFile =~ s/\n$/\n$EdEmptyLnHldr/;
      };
    };
    if ($EdHtmlText) {
      $EdFile = &HtmlText($EdFile);
    } elsif ($EdTAMod) {
      $EdFile =~ s/\<(\/textarea\>)/\<$EdTAMod$1/gi;
    };
    print '<HR><CENTER>',"\n",
    '<TABLE BGCOLOR="',"$TableColor\" $FTableParam",'>',"\n",
    '<TR><TD ALIGN="center">',"\n",
    '<TABLE ',"$TableParam",'><TR><TD ALIGN="center">',"\n",
    '<FORM METHOD="POST" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshedit">',"\n",
    '<INPUT TYPE=hidden name="fname" value="',"$filename",'">',"\n",
    '<TEXTAREA name="edfile" cols=',"$EdCols",' rows=',"$EdRows";
    if ($EdWrap) {print " wrap\=$EdWrap";};
    print '>',"$EdFile",'</TEXTAREA>',"\n",
    '</TD></TR><TR><TD ALIGN="center">',"\n",
    '<INPUT TYPE=submit name="edcom" value="  Save   ">',"\n",
    '<INPUT TYPE=submit name="edcom" value=" Save As ">',"\n",
    '<INPUT TYPE=submit name="edcom" value="Save+Exit">',"\n",
    '<INPUT TYPE=reset value="  Reset  "></FORM>',"\n",
    '<FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshedit">',"\n",
    '<INPUT TYPE=hidden name="fname" value="',"$filename",'">',"\n",
    '<INPUT TYPE=submit name="edcom" value=" Reload  ">',"\n",
    '<INPUT TYPE=submit name="edcom" value=" Options ">',"\n",
    '<INPUT TYPE=submit name="edcom" value="  Exit   ">',"\n",
    '</TD></TR></TABLE></TD></TR></TABLE></FORM></CENTER>',"\n";
  };
}

sub PrintEdOptions { # Prints the text editor options menu.

  local($filename) = @_;
  print '<HTML><HEAD><TITLE> Syrine - Text Editor Options</TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor",'"><B> Syrine: Text Editor Options: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    print '<HR><PRE>',&HtmlText($output),'</PRE>',"\n" if ($output ne 'ok');
    (require "$ProgDir/lib/edtestlib.pl" ||
      &SyrineDie("Can\'t require edtestlib\.pl \: $!")) if ($EdTest);
    print &GetHelp('edhlp.htm','Help') if ($EdHelp);
    print '<HR><FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshedit">',"\n",
    '<INPUT TYPE=hidden name="fname" value="',"$filename",'">',"\n",
    '<INPUT TYPE=hidden name="edcom" value="Options">',"\n",
    '<TABLE><TR><TD NOWRAP><B> Window Size: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP> Horizontal (columns): ',"\n",
    '<INPUT name="edcols" size="4" value="',"$EdCols",'"></TD>',"\n",
    '<TD NOWRAP COLSPAN=12></TD>',"\n",
    '<TD NOWRAP> Vertical (rows): ',"\n",
    '<INPUT name="edrows" size="4" value="',"$EdRows",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Line Wrapping: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=radio name="edwrap" value=""';
    unless ($EdWrap =~ m/virtual/i || $EdWrap =~ m/physical/i) {print ' CHECKED';};
    print '>None </TD><TD NOWRAP COLSPAN=2></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=radio name="edwrap" value="virtual"';
    if ($EdWrap =~ m/virtual/i) {print ' CHECKED';};
    print '>Virtual </TD><TD NOWRAP COLSPAN=2></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=radio name="edwrap" value="physical"';
    if ($EdWrap =~ m/physical/i) {print ' CHECKED';};
    print '>Physical </TD></TR></TABLE>',"\n",
    'Note: The line wrapping selection only effects browsers that support this option',"\n",
    '(mainly, Netscape Navigator). The "Virtual" option wraps lines in the display, but',"\n",
    'does not actually insert new lines in the file. The "Physical" option wraps lines',"\n",
    'in the display, and inserts new lines at the wrap points, just as if new-lines',"\n",
    'had been entered.<BR><BR>',"\n",
    '<HR WIDTH="50%"><B> The options below this line are here only to support isoteric',"\n",
    'behavior of some (mostly older) web browsers. In most cases (in particular,',"\n",
    'if you are using version 2.0 or later of either Netscape Navigator or',"\n",
    'Microsoft Internet Explorer) you should not change any of them from their',"\n",
    '(designated) default values. IMPROPER SETTINGS IN THIS SECTION MAY LEAD',"\n",
    'TO FILE CORRUPTION. If you are not sure about your browser, you should',"\n",
    'take our',"\n",
    '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",
    '&com=rshedit&fname=',&UrlEncode($filename),'&edcom=Options&opcom=EdTest">',"\n",
    'Test of browser &lt;TEXTAREA&gt; behavior. </A> If you modify any of',"\n",
    'these settings in order to temporarily support an older browser, you should',"\n",
    'not forget to change them back to their default values afterwards. Click',"\n",
    '"Help" below for more information. </B><HR WIDTH="50%">',"\n",
    '<BR><TABLE><TR>',"\n",
    '<TD NOWRAP><B> Place Holder for Empty Lines: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="lnhldr" value="',"\'$EdEmptyLnHldr\'",'"></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD>(Default Value: \' \')</TD></TR></TABLE>',"\n",
    'Note: The above place holder can be almost any string of viewable characters',"\n",
    'enclosed in single quotes. (It cannot include any single quotes or consist only of',"\n",
    '0 characters.) This string will be added to empty lines when files are opened with',"\n",
    'the Text Editor. It will be removed when the Text Editor saves a file--if it is the',"\n",
    'only content of a line. This mechanism is used to preserve empty lines, since some',"\n",
    'browsers tend to strip them. In most cases, a single white space should make',"\n",
    'a good place holder.',"\n",
    '<BR><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Empty Line Protection Mode: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=checkbox name="opfl" value="1"';
    if ($EdOPFL) {print ' CHECKED';}; 
    print '> Only Protect First Line </TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD>(Default Value: CHECKED)</TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Text Formatting: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=checkbox name="htmtxt" value="1"';
    if ($EdHtmlText) {print ' CHECKED';}; 
    print '> Encode Special Characters </TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD>(Default Value: CHECKED)</TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> &lt/TEXTAREA&gt Modifier: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="tamod" value="',"\'$EdTAMod\'",'"></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD>(Default Value: \'/\')</TD></TR></TABLE>',"\n",
    'Note: The above modifier can be almost any string of viewable characters',"\n",
    'enclosed in single quotes. (It cannot include any single quotes or consist only of',"\n",
    '0 or &lt characters.) This string will be added to follow the first &lt in any',"\n",
    '&lt/TEXTAREA&gt tag when files are opened with the Text Editor. It will be removed',"\n",
    'from such tags when the Text Editor saves a file. This mechanism enables editing',"\n",
    'of HTML files containing &lt/TEXTAREA&gt tags. It is active only if the',"\n",
    '\'Encode Special Characters\' box above is unchecked. <BR><BR><BR>',"\n",
    '<CENTER><TABLE><TR><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="    OK    "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Cancel  "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Apply   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=reset value="  Reset   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="   Help   "></TD>',"\n",
    '</TR></TABLE></CENTER></FORM>',"\n";    
  };
}

1; # return true
