# fslib-unix.pl - File manipulation subroutines
# This file is part of Syrine 0.1b
# Copyright (C) 1997,1998  Yoram Last (ylast@mindless.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

sub MakeDir { # Creates a new directory.

  local($dirname) = @_;
  mkdir $dirname, 0777;
}

sub DelTree { # Deletes a file or a directory tree.

  local($file) = @_;
  if (-d $file) {
    &RmDirTree($file) || &SyrineDie("Can\'t remove $file \: $!");
  } else {
    unlink($file) || &SyrineDie("Can\'t delete $file \: $!");
  };
  return 1;
}

sub RmDirTree { # Deletes a directory tree.

  local($dir) = @_;
  opendir(DIR,$dir) || &SyrineDie("Can\'t open $dir \: $!");
  local(@items) = grep !/^\.\.?$/, readdir DIR;
  close DIR;
  local($i);
  foreach $i (0..$#items) {
    if (-d "$dir/$items[$i]" && $items[$i] ne '..') {
      &RmDirTree("$dir/$items[$i]") || &SyrineDie("Can\'t remove $dir/$items[$i] \: $!");
    } else {
      unlink("$dir/$items[$i]") || &SyrineDie("Can\'t remove $dir/$items[$i] \: $!");
    };
  };
  rmdir "$dir";
}

sub FCopy { # Copies a file.

  local($file,$dest) = @_;
  local($buf) = '';
  $dest =~ s/[\\\/]$//;
  if (-d $dest) {
    $dest = join('/',$dest,&FileName($file));
  };
  open (FILE,"<$file") || &SyrineDie("Can\'t open $file: $!");
  binmode FILE;
  open (DEST,">$dest") || &SyrineDie("Can\'t write $dest: $!");
  binmode DEST;
  while (read(FILE,$buf,$BufSize)) {print DEST $buf;};
  close FILE; close DEST;
  chmod int((stat $file)[2]), $dest;
  return -e $dest;
}

sub DCopy { # Copies a directory tree.

  local($dir,$dest) = @_;
  $dest =~ s/[\\\/]$//;
  $dir =~ s/[\\\/]$//;
  -e $dir || &SyrineDie("Can\'t find $dir \: $!");
  -d $dir || &SyrineDie("$dir is not a directory");
  if (-e $dest) {
    unless (-d $dest) {
      &SyrineDie("Can\'t copy $dir to $dest \: Destination exists and is not a directory");
    };
  } else {
    mkdir $dest, 0600;
    chmod 0600, $dest;
  };
  opendir(DIR,$dir) || &SyrineDie("Can\'t open $dir \: $!");
  local(@items) = grep !/^\.\.?$/, readdir DIR;
  close DIR;
  local($i);
  foreach $i (0..$#items) {
    if (-d "$dir/$items[$i]" && $items[$i] ne '..') {
      &DCopy("$dir/$items[$i]","$dest/$items[$i]") || 
        &SyrineDie("Failed to copy $dir/$items[$i] to $dest/$items[$i] \: $!");
    } else {
      &FCopy("$dir/$items[$i]","$dest/$items[$i]") ||
        &SyrineDie("Failed to copy $dir/$items[$i] to $dest/$items[$i] \: $!");
    };
  };
  chmod int((stat $dir)[2]), $dest;
  return -e $dest;
}

sub GetInfo { # Gets "ls -l" information for a file or a directory.

  local($file) = @_;
  $file = "\"$file\"" if ($file);
  $_ = `ls -al $file`;
  return $_;
}

sub SyrineZip{
	local($target)=@_;
	&ExUShellCom("zip -r $target $target",$LoginShell,$ColFilter,1);
};
sub SyrineUnZip{
	local($target)=@_;
	$extension=&FileExt($target);
	if($extension eq "ZIP"){
		&ExUShellCom("unzip $target",$LoginShell,$ColFilter,1);
	}elsif($extension eq "GZ"){
		&ExUShellCom("gunzip $target",$LoginShell,$ColFilter,1);
	}elsif($extension eq "TAR"){
		&ExUShellCom("tar zxf $target",$LoginShell,$ColFilter,1);
	}else{
		&SyrineDie("Unknown Extension");	
	};

};
1; # return true
